import os
import PyMockTurtle
import pytest
import time

@pytest.fixture(scope="module")
def devid():
    return 0x1

@pytest.fixture(scope="module")
def cfg():
    cfg = PyMockTurtle.TrtlConfig()
    cfg.signature = PyMockTurtle.TrtlConfig.TRTL_CONFIG_ROM_SIGNATURE
    cfg.version = 0
    cfg.pad = 0
    cfg.clock_freq = 0
    cfg.flags = 0
    cfg.app_id = 0xd330d330
    cfg.n_cpu = 2
    cfg.smem_size = 0x00002000
    cfg.n_hmq[0] = 2
    cfg.n_hmq[1] = 2
    cfg.n_rmq[0] = 1
    cfg.n_rmq[1] = 1

    return cfg

@pytest.fixture(scope="module", params=range(cfg().n_cpu))
def cpu_idx(request):
    return request.param


@pytest.fixture(scope="module", params=[0x1])
def trtl_device(request):
    dev = PyMockTurtle.TrtlDevice(request.param)
    return dev

@pytest.fixture(scope="module", params=range(cfg().n_cpu))
def trtl_cpu(request, trtl_device):
    trtl_device.cpu[request.param].disable()
    yield trtl_device.cpu[request.param]
    trtl_device.cpu[request.param].disable()

@pytest.fixture(scope="module")
def trtl_cpu0(request, trtl_device):
    trtl_device.cpu[0].disable()
    yield trtl_device.cpu[0]
    trtl_device.cpu[0].disable()

@pytest.fixture(scope="module")
def trtl_shm():
    dev = PyMockTurtle.TrtlDevice(devid())
    return dev.shm

@pytest.fixture(scope="module")
def trtl_firmware_dir():
    return os.path.dirname(os.path.abspath(os.path.realpath(__file__)))

@pytest.fixture(scope="module", params=[2**x for x in range(5)])
def trtl_msg(request):
    """This provides a list of test messages. Each test message contains
    two values (so len = 2): a sequence number, the maximum number in the
    sequence. These two values in the last message are equal
    """
    msg_list = []
    for val in range(request.param):
        msg = PyMockTurtle.TrtlMessage()
        msg.header.len = 2
        msg.payload[0] = val + 1
        msg.payload[1] = request.param
        msg_list.append(msg)
    return msg_list

@pytest.fixture(scope="module")
def trtl_version_exp():
    v = PyMockTurtle.TrtlFirmwareVersion()
    v.fw_id = 0xFEDE
    v.fw_version = 0x1
    v.git_version = 0x12345678

    return v

# @pytest.fixture(scope="module")
# def trtl_hmq():
#    dev = PyMockTurtle.TrtlDevice(devid())   for cpu in dev.cpu:
#        for hmq in cpu.hmq:
#            yield hmq
